<?php
/**
 * Phire CMS
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.TXT.
 * It is also available through the world-wide-web at this URL:
 * http://www.phirecms.org/LICENSE.TXT
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@phirecms.org so we can send you a copy immediately.
 *
 * @category   Phire
 * @package    Phire_Db
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 */

/**
 * Phire_Db
 *
 * @category   Phire
 * @package    Phire_Db
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 * @version    1.1.2
 */

class Phire_Db
{

    /**
     * Rows of multiple return results from a database query
     * in an ArrayObject format.
     * @var array
     */
    public $rows = array();

    /**
     * Database connection
     * @var resource
     */
    public $db = null;

    /**
     * Field names
     * @var array
     */
    protected $_fields = array();

    /**
     * Constructor
     *
     * Instantiate the database object.
     *
     * @return void
     */
    public function __construct()
    {

        $this->db = Moc10_Db::getInstance();

    }

    /**
     * Set method to set the property to the value of _fields[$name].
     *
     * @param  string $name
     * @param  mixed $value
     * @throws Exception
     * @return void
     */
    public function __set($name, $value)
    {

        // Check to see if the field key exists.
        if (!array_key_exists($name, $this->_fields)) {
            throw new Exception("The field '" . $name . "' does not exist.");
        } else {
            $this->_fields[$name] = $value;
        }

    }

    /**
     * Get method to return the value of _fields[$name].
     *
     * @param  string $name
     * @throws Exception
     * @return mixed
     */
    public function __get($name)
    {

        // Check to see if the field key exists.
        if (!array_key_exists($name, $this->_fields)) {
            throw new Exception("The field '" . $name . "' does not exist.");
        } else {
            return $this->_fields[$name];
        }

    }

    /**
     * Return the isset value of _fields[$name].
     *
     * @param  string $name
     * @return boolean
     */
    public function __isset($name)
    {

        return isset($this->_fields[$name]);

    }

    /**
     * Unset _fields[$name].
     *
     * @param  string $name
     * @return void
     */
    public function __unset($name)
    {

        $this->_fields[$name] = null;

    }

    /**
     * Set all the table fields at once.
     *
     * @param  array $fields
     * @throws Exception
     * @return void
     */
    public function setAll($fields = null)
    {

        // If null, clear the fields.
        if (is_null($fields)) {
            $this->_fields = array();
            $this->rows = array();
        // Else, if an array, set the fields.
        } else if (is_array($fields)) {
            $this->_fields = $fields;
            $this->rows[0] = new ArrayObject($fields, ArrayObject::ARRAY_AS_PROPS);
        // Else, throw an exception.
        } else {
            throw new Exception('The parameter passed must be either an array or null.');
        }

    }

    /**
     * Execute a custom SQL query.
     *
     * @param  string $sql
     * @return void
     */
    public function execute($sql)
    {

        // Create the DB connection and execute the query.
        $this->db = Moc10_Db::getInstance();
        $this->db->interface->query($sql);

        if (stripos($sql, 'select') !== false) {

            // Set the return results.
            $this->_setResults();

        } else if (stripos($sql, 'delete') !== false) {

            $this->_fields = array();
            $this->rows = array();

        }

    }

    /**
     * Set the query results.
     *
     * @return void
     */
    protected function _setResults()
    {

        // If there is more than one result returned, create an array of results.
        if ($this->db->interface->numRows() > 1) {
            $this->_fields = array();
            while (($row = $this->db->interface->fetch()) != false) {
                $this->rows[] = new ArrayObject($row, ArrayObject::ARRAY_AS_PROPS);
            }
        // Else, set the _fields array to the single returned result.
        } else {
            while (($row = $this->db->interface->fetch()) != false) {
                $this->_fields = $row;
                $this->rows[0] = new ArrayObject($row, ArrayObject::ARRAY_AS_PROPS);
            }
        }

    }

}
